function love.load()
    world = love.physics.newWorld()
    love.physics.setMeter(60)
    world:setCallbacks(beginContact, endContact, preSolve, postSolve)

    paddle = makePaddle(600, 570, 120, 10, {155, 59, 0}, {170, 100, 57}, world)
    ball = makeBall(700, 300, 10, {155, 122, 0}, {170, 46, 57}, world)
    ball.body:setLinearVelocity(10, 150)
    walls = {}
    walls.left = makeWall(0, 300, 10, 600, {80, 80, 200}, world)
    walls.top = makeWall(400, 0, 800, 10, {80, 80, 200}, world)
    walls.right = makeWall(800, 300, 10, 600, {80, 80, 200}, world)
    bricks = {}
    bricks[1] = makeBrick(700, 350, 600, 16, {60, 60, 60}, {40, 40, 40}, world)

    for _, body in pairs(world:getBodyList()) do
        for _, fix in pairs(body:getFixtureList()) do
            fix:setFriction(0)
        end
    end
end


function love.update(dt)
    paddle_move()
    world:update(dt)
end

function love.draw()
    drawPolygonPiece(paddle)
    for _, w in pairs(walls) do
        drawPolygonPiece(w)
    end
    for _, b in pairs(bricks) do
        drawPolygonPiece(b)
    end
    drawCirclePiece(ball)
    love.graphics.print(ball.body:getX(), 0, 0)
    love.graphics.print("velocity: " .. ball.body:getLinearVelocity(), 0, 20)
end

function beginContact(a, b, coll)
    print("contact")
end

function endContact(a, b, coll)

end

function preSolve(a, b, coll)

end

function postSolve(a, b, coll, normalimpulse1, tangentimpulse1, normalimpulse2, tangentimpulse2)
    -- we won't do anything with this function
end

function drawCirclePiece(b)
    love.graphics.setColor(b.color_inner)
    love.graphics.circle("fill", b.body:getX(), b.body:getY(), b.radius, 20)
    love.graphics.setColor(b.color_border)
    love.graphics.circle("line", b.body:getX(), b.body:getY(), b.radius+1, 20)
end

function drawPolygonPiece(p)
    love.graphics.setColor(p.color_inner)
    love.graphics.polygon("fill", p.body:getWorldPoints(p.shape:getPoints()))
    love.graphics.setColor(p.color_border)
    love.graphics.polygon("line", p.body:getWorldPoints(p.shape:getPoints()))

end

function makeBrick(x, y, width, height, color_border, color_inner, world)
    brick = {}
    brick.color_border = color_border
    brick.color_inner = color_inner
    brick.body = love.physics.newBody(world, x, y, "static")
    brick.shape = love.physics.newRectangleShape(width, height)
    brick.fixture = love.physics.newFixture(brick.body, brick.shape, 1)
    return brick
end

function paddle_move()
    move = 0
    if love.keyboard.isDown("left")
    then move = move - 5 end
    if love.keyboard.isDown("right")
    then move = move + 5 end
    paddle.body:setX(paddle.body:getX() + move)
end


function makeBall(x, y, radius, color_border, color_inner, world)
    ball = {}
    ball.radius = radius
    ball.color_border = color_border
    ball.color_inner = color_inner
    ball.body = love.physics.newBody(world, x, y, "dynamic")
    ball.shape = love.physics.newCircleShape(0, 0, ball.radius)
    ball.fixture = love.physics.newFixture(ball.body, ball.shape, 1)
    ball.fixture:setRestitution(1)
    return ball
end

function makePaddle(x, y, width, height, color_border, color_inner, world)
    paddle = {}
    paddle.color_border = color_border
    paddle.color_inner = color_inner
    paddle.body = love.physics.newBody(world, x, y, "kinematic")
    paddle.shape = love.physics.newRectangleShape(0, 0, width, height, 0)
    paddle.fixture = love.physics.newFixture(paddle.body, paddle.shape, 1)
    paddle.fixture:setRestitution(1)
    return paddle
end

function makeWall(x, y, width, height, color, world)
    wall = {}
    wall.color_border = color
    wall.color_inner = color
    wall.body = love.physics.newBody(world, x, y, "static")
    wall.shape = love.physics.newRectangleShape(0, 0, width, height, 0)
    wall.fixture = love.physics.newFixture(wall.body, wall.shape)
    return wall
end

